@@ -17,7 +17,7 @@
 
 #ifndef KOBBY_NOTEPLUGIN_H
 #define KOBBY_NOTEPLUGIN_H
-#include "kobbycommon_export.h"
+#include "ktecollaborative_export.h"
 
 #include <libqinfinity/session.h>
 #include <libqinfinity/noteplugin.h>
@@ -33,7 +33,7 @@ class DocumentBuilder;
 /**
  * @brief Instantiates InfText sessions.
  */
-class KOBBYCOMMON_EXPORT NotePlugin
+class KTECOLLABORATIVECOMMON_EXPORT NotePlugin
     : public QInfinity::NotePlugin
 {
 
